@@ -137,7 +137,7 @@ execute(struct op * volatile t, volatile int flags, volatile int * volatile xerr
 				 */
 				if (tp && tp->type == CSHELL
 				    && (tp->flag & SPEC_BI))
-					errorf(null);
+					errorf(NULL);
 				/* Deal with FERREXIT, quitenv(), etc. */
 				goto Break;
 			}
