@@ -56,6 +56,14 @@
 # include <memory.h>
 #endif
 
+#ifndef SIZE_MAX
+#ifdef SIZE_T_MAX
+#define SIZE_MAX	SIZE_T_MAX
+#else
+#define SIZE_MAX	((size_t)-1)
+#endif
+#endif
+
 #include <stdarg.h>
 #define SH_VA_START(va, argn) va_start(va, argn)
 
@@ -283,54 +291,49 @@ union mksh_ccphack {
 };
 
 /*
- * Area-based allocation built on malloc/free
+ * simple grouping allocator
  */
 
-typedef struct Area {
-	struct link *freelist;	/* free list */
-} Area;
+/* 1. internal structure */
+struct lalloc {
+	struct lalloc *next;
+};
 
-EXTERN	Area	aperm;		/* permanent object space */
-#define	APERM	&aperm
-#define	ATEMP	&e->area
+/* 2. sizes */
+#define ALLOC_ITEM	struct lalloc
+#define ALLOC_SIZE	(sizeof(ALLOC_ITEM))
 
-#ifdef MEM_DEBUG
-# include "chmem.h" /* a debugging front end for malloc et. al. */
-#endif /* MEM_DEBUG */
+/* 3. group structure (only the same for lalloc.c) */
+typedef struct lalloc Area;
 
-#ifdef KSH_DEBUG
-# define kshdebug_init()	kshdebug_init_()
-# define kshdebug_printf(a)	kshdebug_printf_ a
-# define kshdebug_dump(a)	kshdebug_dump_ a
-#else /* KSH_DEBUG */
-# define kshdebug_init()
-# define kshdebug_printf(a)
-# define kshdebug_dump(a)
-#endif /* KSH_DEBUG */
 
+EXTERN Area aperm;		/* permanent object space */
+#define APERM	&aperm
+#define ATEMP	&e->area
 
 /*
  * parsing & execution environment
  */
-EXTERN	struct env {
-	short	type;			/* enviroment type - see below */
-	short	flags;			/* EF_* */
-	Area	area;			/* temporary allocation area */
-	struct	block *loc;		/* local variables and functions */
-	short  *savefd;			/* original redirected fd's */
-	struct	env *oenv;		/* link to previous enviroment */
-	ksh_jmp_buf jbuf;		/* long jump back to env creator */
-	struct temp *temps;		/* temp files */
+extern struct env {
+	ALLOC_ITEM __alloc_i;	/* internal, do not touch */
+	Area area;		/* temporary allocation area */
+	struct env *oenv;	/* link to previous environment */
+	struct block *loc;	/* local variables and functions */
+	short *savefd;		/* original redirected fds */
+	struct temp *temps;	/* temp files */
+	sigjmp_buf jbuf;	/* long jump back to env creator */
+	short type;		/* environment type - see below */
+	short flags;		/* EF_* */
 } *e;
 
 /* struct env.type values */
-#define	E_NONE	0		/* dummy enviroment */
-#define	E_PARSE	1		/* parsing command # */
-#define	E_FUNC	2		/* executing function # */
-#define	E_INCL	3		/* including a file via . # */
-#define	E_EXEC	4		/* executing command tree */
-#define	E_LOOP	5		/* executing for/while # */
-#define	E_ERRH	6		/* general error handler # */
+#define E_NONE	0	/* dummy environment */
+#define E_PARSE	1	/* parsing command # */
+#define E_FUNC	2	/* executing function # */
+#define E_INCL	3	/* including a file via . # */
+#define E_EXEC	4	/* executing command tree */
+#define E_LOOP	5	/* executing for/while # */
+#define E_ERRH	6	/* general error handler # */
 /* # indicates env has valid jbuf (see unwind()) */
 
 /* struct env.flag values */
@@ -344,17 +347,16 @@ EXTERN	struct env {
 /* Do returns stop at env type e? */
 #define STOP_RETURN(t)	((t) == E_FUNC || (t) == E_INCL)
 
-/* values for ksh_siglongjmp(e->jbuf, 0) */
-#define LRETURN	1		/* return statement */
-#define	LEXIT	2		/* exit statement */
-#define LERROR	3		/* errorf() called */
-#define LLEAVE	4		/* untrappable exit/error */
-#define LINTR	5		/* ^C noticed */
-#define	LBREAK	6		/* break statement */
-#define	LCONTIN	7		/* continue statement */
-#define LSHELL	8		/* return to interactive shell() */
-#define LAEXPR	9		/* error in arithmetic expression */
-
+/* values for siglongjmp(e->jbuf, 0) */
+#define LRETURN	1	/* return statement */
+#define LEXIT	2	/* exit statement */
+#define LERROR	3	/* errorf() called */
+#define LLEAVE	4	/* untrappable exit/error */
+#define LINTR	5	/* ^C noticed */
+#define LBREAK	6	/* break statement */
+#define LCONTIN	7	/* continue statement */
+#define LSHELL	8	/* return to interactive shell() */
+#define LAEXPR	9	/* error in arithmetic expression */
 
 /* option processing */
 #define OF_CMDLINE	0x01	/* command line */
@@ -627,6 +629,9 @@ EXTERN	int	x_cols I__(80);	/* tty columns */
 /* user and system time of last j_waitjed job */
 EXTERN struct timeval j_usrtime, j_systime;
 
+
+#define alloc(n, ap)    aresize(NULL, (n), (ap))
+
 /* be sure not to interfere with anyone else's idea about EXTERN */
 #ifdef EXTERN_DEFINED
 # undef EXTERN_DEFINED
