@@ -23,7 +23,7 @@ CPP="${1-cc -E}"
 	{ QwErTy SIG\1 , "\1", "\2", 0, 0, 0, 0, 0 },\
 #endif/') > $in
 $CPP $in  > $out
-sed -n 's/{ QwErTy/{/p' < $out | awk '{print NR, $0}' | sort -k 3n,1n |
+sed -n 's/{ QwErTy/{/p' < $out | awk '{print NR, $0}' | sort -n -k 3 -k 1 |
     sed 's/^[0-9]* //' |
     awk 'BEGIN { last=0; nsigs=0; }
 	{
